#include "DoubleTriggerNoiseAlgorithm.h"
#include "AnalysisAlgorithmMap.h"
#include "AnalysisService.h"
#include "SctData/DoubleTriggerNoiseTestResult.h"
#include "SctData/StandardDefects.h"
#include "SctData/RawScanResult.h"
#include "SctData/ConfigurationVariable.h"
#include "Sct/UnsupportedOperationError.h"
#include "SctData/OccupancyProjector.h"
#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <TH1.h>

using namespace std;
using namespace boost;
using namespace SctData;
using namespace Sct;

namespace SctAnalysis {
    
    DoubleTriggerNoiseAlgorithm::~DoubleTriggerNoiseAlgorithm() throw() {}

    shared_ptr<AnalysisAlgorithm> DoubleTriggerNoiseAlgorithm::clone(const TestData& testData, const string& moduleName) const throw() {
	return shared_ptr<AnalysisAlgorithm>(new DoubleTriggerNoiseAlgorithm(testData, moduleName, *this));
    }

    bool DoubleTriggerNoiseAlgorithm::inMap = AnalysisAlgorithmMap::instance().setAlgorithm("DoubleTriggerNoiseTest", auto_ptr<AnalysisAlgorithm>(new DoubleTriggerNoiseAlgorithm()));

    //----------------------------------------------------------------------
    
    void DoubleTriggerNoiseAlgorithm::loadData() {
	loadAllRaws();
    }
    
    bool DoubleTriggerNoiseAlgorithm::canAnalyze() const {
	return hasAllRaws();
    }
    
    shared_ptr<SctData::TestResult> DoubleTriggerNoiseAlgorithm::createTestResult() const {
	return shared_ptr<TestResult>(new DoubleTriggerNoiseTestResult());
    }

    void DoubleTriggerNoiseAlgorithm::analyze() {
	shared_ptr<DoubleTriggerNoiseTestResult> result = dynamic_pointer_cast<DoubleTriggerNoiseTestResult>(getTestResult());
	const SctData::ConfigurationVariable& scanVariable = getRaw(0)->getHeader().getVariable();
	result->setScanVariable(scanVariable);
	const ModuleConfiguration config=getRaw(0)->getConfiguration();
	
	m_scanVariable = SctData::ConfigurationVariableIOHelper::getTypeRep(scanVariable);
	
	result->setPassed(true);

	OccupancyProjector opr(*getRaw(0) );
	for (unsigned int ichip=0; ichip<nChipModule; ++ichip) {
	  //const ChipConfiguration& chipconfig=config.getChipConfiguration(ichip);
	  //DefectList& defects = result->getDefects();
	  auto_ptr<TH1> occ = opr.getOccupancy("temp", ModuleElement::Chip(ichip)) ;
	  float peak=0; // maximum occupancy over time bins
	  float av=0;   // average occupancy over time bins
	  for (unsigned ibin=1; ibin<=occ->GetNbinsX(); ++ibin){
	    const float value=occ->GetBinContent(ibin);
	    if (value>peak) peak=value;
	    av+=value;
	  }
	  if (occ->GetNbinsX()>0) av/=static_cast<float>(occ->GetNbinsX());
	  result->setPeakOccupancy(ichip, peak);
	  result->setPedestalOccupancy(ichip, av);
	  if (peak > 5E-4 || peak > av*5.) result->setPassed(false);
	}
    }
} // end of namespace SctAnalysis
