#include "FitterInterface.hh"
#include "FitterArguments.h"

namespace SctFitter {
  using std::string;
  using std::list;
  using std::endl;

  FitterArguments::FitterArguments(int argc, char** argv) : 
    Arguments(argc, argv), m_strategy("NagFitStrategy"), m_options("NQR"), m_analytic(false)
  {
    parse();
  }
  void FitterArguments::parse(){
    Arguments::parse();
    for (list<string>::iterator i=m_list.begin(); i!=m_list.end(); ++i){
      if (*i == "-analytic"){
	m_analytic=true;
      }else if (*i == "-options"){
	m_options=next(i);
      } else if (*i == "-strategy"){
	m_strategy=next(i);
      }
    }
    if (instanceName()=="") setInstanceName(FitterI::FitterInterface::instanceName);
  }
  void FitterArguments::printHelp(std::ostream& oss) const{
    oss << "FitterArguments:" << endl
	<< "  -strategy  <strategy name>     : set fit stragety to e.g. RootFitStrategy or NagFitStrategy" <<endl
	<< "  -options  <options>            : set fit options e.g. NQR" <<endl
	<< "  -analytic (switch)             : switch on analytic fitting algorithm" << endl;
    Arguments::printHelp(oss);
  }

  std::string FitterArguments::getStrategyName() const{
    return m_strategy;
  }

  std::string FitterArguments::getFitOptions() const{
    return m_options;
  }
  
  bool FitterArguments::analytic() const{
    return m_analytic;
  }
}
