
#ifndef SCT_SPECIALSUBSCRIBEREQUEST_H
#define SCT_SPECIALSUBSCRIBEREQUEST_H

#include "SpecialSubscribeRequest.fwd"

#include "SubscriptionManagerImpl.fwd"
#include "NameOrRegexp.h"

std::ostream & operator<<(std::ostream & os, const Sct::SpecialSubscribeRequest & ssr);

namespace Sct {
  
  class SpecialSubscribeRequest {
    // we are going to prevent anyone except SubscriptionManagerImpl from making us
    friend class SubscriptionManagerImpl;
    friend std::ostream & ::operator<<(std::ostream & os, const Sct::SpecialSubscribeRequest & ssr);
  private:
    SpecialSubscribeRequest(const NameOrRegexp & serverPattern,
                            const NameOrRegexp & objectPattern,
                            const is::callback_ptr cb) :
      m_serverPattern(serverPattern),
      m_objectPattern(objectPattern),
      m_cb(cb) {
    };
    std::ostream & printMeTo(std::ostream & os) const {
      return os << "SpecialSubscribeRequest["
                << "Server[" << m_serverPattern << "]"
                << ", " 
                << "Object[" << m_objectPattern << "]"
                << "]";
    };
    const NameOrRegexp m_serverPattern;
    const NameOrRegexp m_objectPattern;
    const is::callback_ptr m_cb;
  };

};

inline std::ostream & operator<<(std::ostream & os, const Sct::SpecialSubscribeRequest & ssr) {
  return ssr.printMeTo(os);
};

#endif
