

#include "LargeCarrot.h"
#include "LargeOnion.h"
#include <ipc/core.h>
#include <unistd.h>


void doStuff(const unsigned int i) {
  std::cout << "Starting doStuff for the " << i << "-th time ... " << std::endl;
  std::cout << "Making a carrot ..." << std::endl;
  LargeCarrot & largeCarrot = *(new LargeCarrot); // not supposed to do "LargeCarrot largeCarrot;" as this can lead to largeCarrot being deleted twice -- once when largeCarrot goes out of scope, and once if/when someone remotely requests it be deleted.".  For this reason, we are equally only supposed to delete it with _destroy();

  //std::cout << "Making an onion ..." << std::endl;
  //LargeOnion & largeOnion = *(new LargeOnion);

  std::cout << "Waiting for carrot to finish blocking ..." << std::endl;
  largeCarrot.run();
  std::cout << "Quiting as carrot finished blocking ..." << std::endl;

  std::cout << "Calling largeCarrot._destroy() ... " << std::endl;
  largeCarrot._destroy();

  // std::cout << "Calling largeOnion._destroy() ... " << std::endl;
  //largeOnion._destroy();

  std::cout << "... finishing doStuff" << std::endl;
};

int main(int argc, char * argv[]) {
  std::cout << "Before I will do much that is of use you will need to run the following two commands:\n" << std::endl;
  std::cout << "\tipc_server &\n\tipc_server -p SCT &\n\n";
  std::cout << "which will start up the necessary partitions into which the objects created in this file will be published.\n" << std::endl;
  std::cout << "NB.  Should you want to do so, you could remotely shut down the onions and carrots with:\n\n\tipc_rm -p SCT --interface Stew/Onion --name LargeOnionName\n\tipc_rm -p SCT --interface Stew/Carrot --name LargeCarrotName\n\n" << std::flush;

  std::cout << "Initialising IPC ..." << std::endl;
  IPCCore::init(argc, argv);

  std::cout << "Making SCT partition..." << std::endl;
  IPCPartition partition("SCT");

  for(unsigned int i=0; i<500; ++i) {
    doStuff(i);
  };

  const unsigned int s=0;
  std::cout << "Sleeping for " << s << " seconds." << std::endl;
  sleep(s);

  std::cout << "Bye" << std::endl;
};
