#include "CorbaExceptionWrapper.h"

namespace Sct {

//CorbaExceptionWrapper methods

 std::string CorbaExceptionWrapper::whatDoesThisCORBAExceptionTellUs(const CORBA::Exception & error) throw() {
    
    try {
      std::ostringstream os;
      os << "CorbaExceptionWrapper[";
      
      os << "_name=[";
      try {
        os << error._name() << "], ";
      } catch(...) {
        os << "oops], ";
      };
      
      // See if it was a system exception
      const CORBA::SystemException * sysEx = dynamic_cast<const CORBA::SystemException*>(&error);
      {
        if (sysEx) {
          os << "is a CORBA::SystemException with ";
          os << "NP_minorString=["; // this part is omniORB specific ... not in all CORBAs.
          try {
	    const std::string minorString = sysEx->NP_minorString(); 
            os << minorString << "], ";
            if (minorString=="TRANSIENT_CallTimedout") {
	       os << "(sometimes you see this message because one of your CORBA methods took longer to return than the interval in milliseconds defined by TDAQ_IPC_TIMEOUT which is read by Sergei's ipc package, part of the TDAQ online software.  See http://www.hep.phy.cam.ac.uk/daq-bin/wiki.cgi/CorbaExceptions or grep for  TDAQ_IPC_TIMEOUT in the online software.  Last seen in online-00-22-00/installed/../ipc/ipc-04-00-09/src/core.cc.  Fix the core problem by making your pc calls non blocking, or work around by adding TDAQ_IPC_TIMEOUT to the environment of the program.  See diff between versions 1.2 and 1.3 of config/databases/SCT_core.data.xml for example.), ";
            };
          } catch(...) {
            os << "oops], ";
          };
          
          os << "minorCode=["; // now we are back onto generic CORBA methods!
          try {
            os << sysEx->minor() << "]";
          } catch(...) {
            os << "oops]";
          };
        } else {
          os << "end";
        };
      }
      os << ".";
    
      return os.str();

    } catch (...) {

      return "Oh dear ... something broke while trying to find out what this CORBA::Exception was made of so I can't tell you anything about it!"; 

    };
  };

  CorbaExceptionWrapper::CorbaExceptionWrapper(const CORBA::Exception& error, const string& msg, const string& file, int line) throw() {
    
    try {
      std::ostringstream os;
      
      os << "CorbaExceptionWrapper created with requested message=["<<msg<<"] and requested FILE=["<<file<<"] and requested line=["<<line<<"].  In addition the wrapped CORBA::Exception reports: [" << whatDoesThisCORBAExceptionTellUs(error) << "]";
      
      const std::string longMessage = os.str();
      initialize("CORBA_EXCEPTION", "Sct::CorbaExceptionWrapper", longMessage, 0, file, line);
      
      name=error._name();
      
    } catch (...) {
      
      initialize("CORBA_EXCEPTION", "Sct::CorbaExceptionWrapper", "Something terrible happened while wrapping whatever this exception was so I can't tell you much about it!", 0, file, line);
      
      name="Sorry, not known!";
      
    };
  }

CorbaExceptionWrapper::~CorbaExceptionWrapper() throw() {}

shared_ptr<Throwable> CorbaExceptionWrapper::clone() const throw() {
    return shared_ptr<Throwable>(new CorbaExceptionWrapper(*this));
}

string CorbaExceptionWrapper::getMessage() const throw() {
    return m_name;
}


}
