#include "../DoubleTriggerNoiseTestResult.h"
#include "DoubleTriggerNoiseTestResultStreamer_v2.h"
#include <vector>

using namespace Sct;

namespace SctData {
namespace IO {

/* READ NOTES ON STREAMERS AND VERSIONS BEFORE EDITING THIS FILE! */
unsigned DoubleTriggerNoiseTestResultStreamer_v2::s_version=2;
	
DoubleTriggerNoiseTestResultStreamer_v2::DoubleTriggerNoiseTestResultStreamer_v2() throw() {
}

bool DoubleTriggerNoiseTestResultStreamer_v2::inMap = IOManager::addToMap("SctData::DoubleTriggerNoiseTestResult",  auto_ptr<Streamer>(new DoubleTriggerNoiseTestResultStreamer_v2()));

shared_ptr<Streamable> DoubleTriggerNoiseTestResultStreamer_v2::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> ad (new DoubleTriggerNoiseTestResult());
    read(in, *ad, manager);
    return ad;
}

void DoubleTriggerNoiseTestResultStreamer_v2::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Let superclass do its work
    manager.writeImpl(out, ob, "SctData::TestResult");
    const DoubleTriggerNoiseTestResult& result=dynamic_cast<const DoubleTriggerNoiseTestResult&>(ob);
    
    float* vals=new float[nChipModule];
    for (unsigned ichip=0; ichip<nChipModule; ++ichip){
      vals[ichip]=result.getPedestalOccupancy(ichip);
    }
    out.put(vals,nChipModule);
    
    for (unsigned ichip=0; ichip<nChipModule; ++ichip){
      vals[ichip]=result.getPeakOccupancy(ichip);
    }
    out.put(vals,nChipModule);
    delete[] vals;
}

void DoubleTriggerNoiseTestResultStreamer_v2::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    manager.readImpl(in, ob, "SctData::TestResult");
    DoubleTriggerNoiseTestResult& result=dynamic_cast<DoubleTriggerNoiseTestResult&>(ob);

    float* peaks;
    float* peds;

    unsigned length;
    in.get(&peds, length);
    if (length!=nChipModule) throw StreamCorruptedException("Wrong size of data",__FILE__, __LINE__);
    for (unsigned ichip=0; ichip<nChipModule; ++ichip){
      result.setPedestalOccupancy(ichip, peds[ichip]);
    }
    
    in.get(&peaks, length);
    if (length!=nChipModule) throw StreamCorruptedException("Wrong size of data",__FILE__, __LINE__);
    for (unsigned ichip=0; ichip<nChipModule; ++ichip){
      result.setPeakOccupancy(ichip, peaks[ichip]);
    }
    delete[] peds;
    delete[] peaks;
}

}
}
