
#ifndef SCTAPIIPC_SCANWRAPPER_H
#define SCTAPIIPC_SCANWRAPPER_H

/** @class ScanWrapper ScanWrapper.h
 *
 * The class ScanWrapper is designed to look externally like a
 * ::SctApi::Scan (i.e. implement the ::SctApiScan interface) 
 * while actually just passing all these calls through to a CORBA 
 * scan object, which is itself internally implemented by a 
 * "standard" SctApi scan object!
 *
 * This class DOES NOT OWN the object reference which it is 
 * constructed with!  An alternative strategy could be developed ...
 *
 * @author Christopher Lester
 **/

//Forward declaration
class ScanWrapper;

// Need complete types for public inheritence
#include "SctApi/SctApi.h" // for ::SctApi::Scan interface
#include "Sct_SctApi/SctApi.hh" // for Scan{_var,_ptr} interface 

class ScanWrapper : public ::SctApi::Scan {
 public:
  typedef enum MemoryPolicy_enum { 
    takeOwnershipOfObjectReference, 
    doNotTakeOwnershipOfObjectReference
  } MemoryPolicy;
 private:
  const MemoryPolicy m_memoryPolicy;
 private:
  Sct_SctApi::Scan_var m_worker;
  Sct_SctApi::Scan_ptr corbaScanServer() const { return Sct_SctApi::Scan::_duplicate(m_worker); };
  Sct_SctApi::Scan_ptr corbaScanServer() { return Sct_SctApi::Scan::_duplicate(m_worker); };
 public:
  typedef ::SctApi::Scan::ScanPoints ScanPoints;
  typedef ::SctApi::Scan::TrigPoints TrigPoints;
  typedef ::SctApi::Scan::ScanOptionsEnum ScanOptionsEnum;
 public:
  ScanWrapper(Sct_SctApi::Scan_ptr w/*, const MemoryPolicy memoryPolicy*/) :
    m_memoryPolicy(doNotTakeOwnershipOfObjectReference/*memoryPolicy*/) {
    if (m_memoryPolicy == doNotTakeOwnershipOfObjectReference) {
      m_worker = Sct_SctApi::Scan::_duplicate(w);
    } else {
      m_worker = w;
    };
  };
  virtual ~ScanWrapper() {
  };

  void configure(unsigned short type, float start, float stop, float step) { return corbaScanServer()->configure(type, start, stop, step); };
  void configure2(unsigned short type, float start, float stop, float step) { return corbaScanServer()->configure2(type, start, stop, step); };

  const boost::shared_ptr< ::SctApi::Trigger> getTrigger1() const;
  const boost::shared_ptr< ::SctApi::Trigger> getTrigger2() const;

  boost::shared_ptr< ::SctApi::Trigger> getTrigger1();
  boost::shared_ptr< ::SctApi::Trigger> getTrigger2();

  void setTrigger1(boost::shared_ptr< ::SctApi::Trigger> trigger);
  void setTrigger2(boost::shared_ptr< ::SctApi::Trigger> trigger);

  void setScanVariable1(unsigned short var) { return corbaScanServer()->setScanVariable1(var); };
  void setScanVariable2(unsigned short var) { return corbaScanServer()->setScanVariable2(var); };

  unsigned short getScanVariable1() const { return corbaScanServer()->getScanVariable1(); };
  unsigned short getScanVariable2() const { return corbaScanServer()->getScanVariable2(); };

  void setNTrigs(unsigned long nTrigs) { corbaScanServer()->setNTrigs(nTrigs); };
  unsigned long getNTrigs() const { return corbaScanServer()->getNTrigs(); };

  const TrigPoints getVariableTrigs() const;
  void setVariableTrigs(const TrigPoints &scans);
  void setVariableTrigRange(unsigned short start, unsigned short end, unsigned long value);

  const ScanPoints getScanPoints1() const;
  const ScanPoints getScanPoints2() const;
  void setScanPoints1(const ScanPoints &scans);
  void setScanPoints2(const ScanPoints &scans);

  int getOption(ScanOptionsEnum scanOption) const;
  void setOption(ScanOptionsEnum scanOption, int option);

  /// Print the scan
  void print() const { corbaScanServer()->print(); };

  void setScanNumber(unsigned int scan);
  unsigned int getScanNumber() const;
  void setRunNumber(unsigned int scan);
  unsigned int getRunNumber() const;

  ::SctApi::ModuleList getModuleList(unsigned int group) const;
  void setModuleList(unsigned int group, const ::SctApi::ModuleList & newList);
  unsigned int getNGroups() const;
  void setNGroups(unsigned int val);
  
  boost::posix_time::ptime getStartTime() const;
  void setStartTime(boost::posix_time::ptime t);
  
  boost::posix_time::ptime getEndTime() const;
  void setEndTime(boost::posix_time::ptime t);
};

#endif
