#include <map>

#include <boost/shared_ptr.hpp>

#include <ipc/object.h>
#include <is/isinfo.h>
#include <mrs/message.h>
#include <omniORB4/poa.h> // For PortableServer::RefCountServantBase

namespace SctApi {
  class SctApi;
  class Trigger;
  class TriggerImpl;
  class Scan;
  class ScanDefImpl;
}

#include "Sct_SctApi/SctApi.hh"

/**
   This gets defined in the main program,
   its needed to attach the Scans and Triggers
*/

// Obsolete?
// extern std::auto_ptr<IPCServer> ipcServer; 

class ScanServer;
class TriggerServer;

/**
   The SctApi ILU implementation/wrapper
 */
class SctApiServer : public IPCNamedObject<POA_Sct_SctApi::SctApiIPC> {

 public:

  SctApiServer(SctApi::SctApi& api,
	       const char* interfaceName);
  virtual ~SctApiServer();		// destructor

 // methods

  virtual void initialiseAll (CORBA::ULong runNumber);
  virtual void shutdownAll ();
  virtual void status ();
  virtual void setRunNumber (CORBA::ULong newRun);
  virtual void setScanNumber (CORBA::ULong newScan);
  virtual CORBA::ULong getRunNumber ();
  virtual CORBA::ULong getScanNumber ();
  virtual void unsetDebugOption (const char * opt);
  virtual void setDebugOption (const char * opt);
  virtual Sct_SctApi::DebugOptionList * listEnabledDebugOptions ();
  virtual Sct_SctApi::DebugOptionList * listDebugOptions ();
  virtual void standardRegisterDump (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void standardRegisterDumpAll ();
  virtual sctConf::ModuleList * getModuleList ();
  virtual CORBA::Boolean isRODPresent (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void awaitResponse (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::DataBlock * getResponse (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void configureBOC (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::BOCMonitorSequence* getBOCMonitorArray(CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void flashLED (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::UShort slaveNumber, CORBA::ULong period, CORBA::ULong flashes);
  virtual void echo (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, const Sct_SctApi::DataBlock & data);
  virtual void createDebugPrimList ();
  virtual void addDebugPrimList (CORBA::Long index, CORBA::Long id, CORBA::Long version, const Sct_SctApi::DataBlock & body);
  virtual void sendDebugPrimList (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void sendDebugSlavePrimList (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong slave, CORBA::Boolean await, CORBA::Boolean response);
  virtual void loadConfiguration (const char * filename);
  virtual void storeModuleConfigurations ();
  virtual void configureAllModules ();
  virtual sctConf::ABCDModule retrieveModule (CORBA::ULong mid);
  virtual CORBA::Short dspBlockDump (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long dspStart, CORBA::Long numWords, CORBA::Long dspNumber);
  virtual Sct_SctApi::DataBlock * dspBlockRead (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long dspStart, CORBA::Long numWords, CORBA::Long dspNumber);
  virtual CORBA::Short dspBlockWrite (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, const Sct_SctApi::DataBlock & buffer, CORBA::ULong dspAddress, CORBA::Long dspNumber);
  virtual CORBA::ULong dspSingleRead (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong dspAddr, CORBA::Long dspNumber);
  virtual void dspSingleWrite (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong dspAddr, CORBA::ULong val, CORBA::Long dspNumber);
  virtual Sct_SctApi::DataBlock * readSlaveDsp (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Short s, CORBA::ULong add, CORBA::ULong words);
  virtual CORBA::ULong findModuleSerial (const char * sn);
  virtual char * convertToString (CORBA::ULong mid);
  virtual CORBA::ULong findModule (CORBA::UShort mur, CORBA::UShort number);
  virtual CORBA::ULong findModuleRod (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel);
  virtual CORBA::ULong findBarrelModule (CORBA::ULong barrel, CORBA::ULong row, CORBA::Short number);
  virtual void convertToBarrelModule (CORBA::ULong mid, CORBA::ULong & barrel, CORBA::ULong & row, CORBA::Short & number);
  virtual CORBA::ULong findEndcapModule (CORBA::Long disk, CORBA::Long ring, CORBA::Long number);
  virtual void convertToEndcapModule (CORBA::ULong mid, CORBA::Long & disk, CORBA::ULong & ring, CORBA::ULong & number);
  virtual void getABCDModules (Sct_SctApi::BankType bank);
  virtual void getABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank);
  virtual sctConf::ABCDModule getABCDModuleRaw (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong slot, Sct_SctApi::BankType bank);
  virtual void setABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank);
  virtual void setABCDModules (Sct_SctApi::BankType bank);
  virtual void sendABCDModule (CORBA::ULong mid, Sct_SctApi::BankType bank, CORBA::ULong type);
  virtual void sendABCDModules (Sct_SctApi::BankType bank);
  virtual void modifyABCDMask (CORBA::ULong mid, const Sct_SctApi::DataBlock & mask);
  virtual void modifyABCDTrims (CORBA::ULong mid, const Sct_SctApi::CharBlock & trims);
  virtual void modifyABCDRC (CORBA::ULong mid, CORBA::ULong chip, CORBA::UShort index, CORBA::Double p0, CORBA::Double p1, CORBA::Double p2);
  virtual void modifyABCDVar (CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDModuleVar (CORBA::ULong mid, CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDChipVar (CORBA::ULong mid, CORBA::ULong chip, CORBA::ULong typ, CORBA::Double var);
  virtual void modifyABCDVarROD (CORBA::ULong typ, CORBA::Double var, Sct_SctApi::BankType bank);
  virtual void modifyABCDModuleVarROD (CORBA::ULong mid, CORBA::ULong typ, CORBA::Double var, Sct_SctApi::BankType bank);
  virtual void modifyBOCParam (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong channel, CORBA::ULong type, CORBA::ULong val);
  virtual void lasersOff ();
  virtual void timSetFrequency (CORBA::ULong partition, CORBA::ULong crate, CORBA::Double trigFreq, CORBA::Double rstFreq);
  virtual void freeTriggers (CORBA::ULong partition, CORBA::ULong crate);
  virtual void stopTriggers (CORBA::ULong partition, CORBA::ULong crate);
  virtual void timL1A (CORBA::ULong partition, CORBA::ULong crate);
  virtual void timCalL1A (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short delay);
  virtual CORBA::Short timReadRegister (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short value);
  virtual void timWriteRegister (CORBA::ULong partition, CORBA::ULong crate, CORBA::Short reg, CORBA::Short value);
  virtual void sendTimTrigger (CORBA::ULong partition, CORBA::ULong crate, Sct_SctApi::Trigger_ptr trig);
  virtual void sendTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig);
  virtual void doScan (Sct_SctApi::Scan_ptr thisScan);
  virtual void doRawScan (Sct_SctApi::Scan_ptr thisScan, CORBA::Long delay, CORBA::Long width, CORBA::Boolean configure, CORBA::Boolean clockByTwo);
  virtual void printBOCSetup (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::BOCChannelList * currentBOCSetup (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual sctConf::BOCGlobalConfig currentBOCRegisters (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void printABCDModule (CORBA::ULong mid);
  virtual void printABCDRodModule (CORBA::ULong mid, Sct_SctApi::BankType bank);
  virtual void decodeEvent (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Short sl, CORBA::Short index, CORBA::Boolean extFlag, CORBA::Boolean errorType);
  virtual void rawData (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long delay, CORBA::Long units, CORBA::Boolean setMask);
  virtual void rawDataWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::Long delay, CORBA::Long units, CORBA::Boolean setMask, Sct_SctApi::Trigger_ptr trig);
  virtual void sendL1A (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::CharBlock * probe (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual Sct_SctApi::CharBlock * probeHarness (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong harness);
  virtual Sct_SctApi::CharBlock * probeWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig);
  virtual Sct_SctApi::CharBlock * probeHarnessWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Trigger_ptr trig, CORBA::ULong harness);
  virtual Sct_SctApi::CharMatrix * probeScanHarnessWithTrigger (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, Sct_SctApi::Scan_ptr trig, CORBA::ULong harness);
  virtual CORBA::Boolean checkAllModulesProbe (const char * chars);
  virtual Sct_SctApi::AutoResultList * autoConfigureScan ();
  virtual CORBA::ULong getRodSlot (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void testLinkOutSelect (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod, CORBA::ULong link);
  virtual CORBA::ULong testLinkOutRetrieve (CORBA::ULong partition, CORBA::ULong crate, CORBA::ULong rod);
  virtual void requestHardReset (CORBA::ULong mid);
  virtual void requestIVCurve (CORBA::ULong mid, CORBA::Float start, CORBA::Float stop, CORBA::Float step, CORBA::UShort delay, CORBA::Float currentLim);
  virtual void setSelect (CORBA::ULong mid, CORBA::Boolean value);
  virtual void setSelectAllInCrate (CORBA::Long crate, CORBA::Boolean value);
  virtual void changeRunState (CORBA::Long state);
  virtual void ddcSetSingleTimeout (CORBA::Short timeout);
  virtual CORBA::Short ddcGetSingleTimeout ();
  virtual void ddcSetMultipleTimeout (CORBA::Short timeout);
  virtual CORBA::Short ddcGetMultipleTimeout ();
  virtual void ddcChangeState (CORBA::Long crate, CORBA::Long channel, CORBA::Short state);
  virtual void ddcChangeStateAll (CORBA::Long crate, CORBA::Short state);
  virtual void ddcLoadConfiguration (CORBA::Long crate, CORBA::Short state);
  virtual void ddcHardReset (CORBA::Long crate, CORBA::Long channel);
  virtual void ddcHardResetAll (CORBA::Long crate);
  virtual CORBA::ULong ddcGetChannelParameterInt (CORBA::Long crate, CORBA::Long channel, const char * name);
  virtual CORBA::Float ddcGetChannelParameterFloat (CORBA::Long crate, CORBA::Long channel, const char * name);
  virtual char * ddcGetChannelParameterString (CORBA::Long crate, CORBA::Long channel, const char * name);
  virtual CORBA::ULong ddcGetCardParameterInt (CORBA::Long crate, CORBA::Long card, const char * name);
  virtual CORBA::Float ddcGetCardParameterFloat (CORBA::Long crate, CORBA::Long card, const char * name);
  virtual char * ddcGetCardParameterString (CORBA::Long crate, CORBA::Long card, const char * name);
  virtual CORBA::ULong ddcGetCrateParameterInt (CORBA::Long crate, const char * name);
  virtual CORBA::Float ddcGetCrateParameterFloat (CORBA::Long crate, const char * name);
  virtual char * ddcGetCrateParameterString (CORBA::Long crate, const char * name);
  virtual void ddcGetCrateParameters (CORBA::Long crate, const Sct_SctApi::NameList & names, Sct_SctApi::NameValueList_out nameValues);
  virtual void ddcSetCrateParameterFloat(CORBA::Long crate, const char* name, CORBA::Float value);
  virtual void ddcSetCardParameterFloat(CORBA::Long crate, CORBA::Long card, const char* name, CORBA::Float value);
  virtual void ddcSetChannelParameterFloat(CORBA::Long crate, CORBA::Long channel, const char* name, CORBA::Float value);
  virtual Sct_SctApi::Scan_ptr createScan ();
  virtual Sct_SctApi::Scan_ptr getDefaultScan ();
  virtual Sct_SctApi::Trigger_ptr createTrigger ();
  virtual Sct_SctApi::Trigger_ptr getDefaultTrigger ();
  virtual Sct_SctApi::ScanList * knownScans ();

 private:
  typedef boost::shared_ptr<ScanServer>  SharedScanServer;

    public:
  typedef std::list<SharedScanServer> Scans;
  Scans scans;

  ScanServer *defaultScan;
  TriggerServer *defaultTrigger;

  static SctApiServer *singleton;

  virtual void shutdown();

 protected:
  SctApi::SctApi &worker();

  MRSStream mrsStream;
  ISInfoDictionary isDict;
  
  CORBA::Short ddcSingleTimeout;
  CORBA::Short ddcMultipleTimeout;

 private:
  SctApi::SctApi& api;
};

// ILU wrapper for SctApi::Scan class
class ScanServer : public IPCNamedObject<POA_Sct_SctApi::Scan>
{
  ScanServer(const ScanServer &);
  ScanServer &operator=(const ScanServer &);

  static int nextNumber;

 public:
  ScanServer();

  //ScanServer(const ScanServer &copy);

  virtual ~ScanServer();

  virtual void print ();
  virtual void configure (CORBA::UShort type, CORBA::Double start, CORBA::Double stop, CORBA::Double step);
  virtual void configure2 (CORBA::UShort type, CORBA::Double start, CORBA::Double stop, CORBA::Double step);
  virtual Sct_SctApi::Trigger_ptr getTrigger1 ();
  virtual Sct_SctApi::Trigger_ptr getTrigger2 ();
  virtual void setTrigger1 (Sct_SctApi::Trigger_ptr trig);
  virtual void setTrigger2 (Sct_SctApi::Trigger_ptr trig);
  virtual void setScanVariable1 (CORBA::UShort var);
  virtual void setScanVariable2 (CORBA::UShort var);
  virtual CORBA::UShort getScanVariable1 ();
  virtual CORBA::UShort getScanVariable2 ();
  virtual void setNTrigs (CORBA::ULong nTrigs);
  virtual CORBA::ULong getNTrigs ();
  virtual Sct_SctApi::Scan::TrigPoints * getVariableTrigs();
  virtual void setVariableTrigs(const Sct_SctApi::Scan::TrigPoints& scans);
  virtual void setVariableTrigRange(CORBA::UShort start, CORBA::UShort end, CORBA::ULong value);
  virtual Sct_SctApi::Scan::ScanPoints * getScanPoints1 ();
  virtual Sct_SctApi::Scan::ScanPoints * getScanPoints2 ();
  virtual void setScanPoints1 (const Sct_SctApi::Scan::ScanPoints & scans);
  virtual void setScanPoints2 (const Sct_SctApi::Scan::ScanPoints & scans);
  virtual CORBA::Long getOption (Sct_SctApi::Scan::ScanOptions opt);
  virtual void setOption (Sct_SctApi::Scan::ScanOptions opt, CORBA::Long val);
  virtual void setScanNumber (CORBA::UShort scan);
  virtual void setRunNumber (CORBA::UShort scan);
  virtual CORBA::ULong getScanNumber ();
  virtual CORBA::ULong getRunNumber ();
  virtual sctConf::ModuleList * getModuleList (CORBA::ULong group);
  virtual void setModuleList (CORBA::ULong group, const sctConf::ModuleList& newList);
  virtual CORBA::ULong getNGroups ();
  virtual void setNGroups(CORBA::UShort val);
  virtual char * getStartTime();
  virtual void setStartTime(const char* t);
  virtual char * getEndTime();
  virtual void setEndTime(const char* t);

  // Override default freeable
#warning LESTER is guessing that the old destroy method corresponds to the new shutdown, so ... there is ALSO a similar method in the ScanServer class which I have similarly altered.
  // LESTER WAS
  // virtual void destroy(ipcStatus *_status);
  // LESTER BECOMES
  virtual void shutdown();

  boost::shared_ptr<SctApi::Scan> m_worker;

 protected:
  static std::string getServerName();

 protected:
  SctApi::Scan &worker();

};

// ILU wrapper for SctApi::Trigger class
class TriggerServer : 
      public IPCNamedObject<POA_Sct_SctApi::Trigger>,
      public virtual PortableServer::RefCountServantBase {
  TriggerServer(const TriggerServer &);
  TriggerServer &operator=(const TriggerServer &);
  static int nextNumber;
 public:
  TriggerServer();
  explicit TriggerServer(const boost::shared_ptr< ::SctApi::Trigger> other);

  //static TriggerServer *getServer(boost::shared_ptr< ::SctApi::Trigger> trigger);
  //static void mapServer(boost::shared_ptr< ::SctApi::Trigger> trigger, TriggerServer *server);

  virtual ~TriggerServer();

  virtual void print ();
  virtual void singleL1A ();
  virtual void doubleL1A (CORBA::UShort delay);
  virtual void delayedL1A (CORBA::UShort delay);
  virtual void calL1A (CORBA::UShort delay);
  virtual void pulseL1A (CORBA::UShort delay);
  virtual void softL1A (CORBA::UShort delay);
  virtual void softCalL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void softPulseL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void softBc (CORBA::UShort delay);
  virtual void bcL1A (CORBA::UShort delay);
  virtual void bcCalL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void bcPulseL1A (CORBA::UShort delay, CORBA::UShort delay2);
  virtual void setCommIncr (CORBA::UShort command, CORBA::UShort incr);
  virtual void getCommIncr (CORBA::UShort & command, CORBA::UShort & incr);

  virtual Sct_SctApi::RODTriggers * getRODTriggers();

  virtual void shutdown();

 private:
  boost::shared_ptr< ::SctApi::Trigger> m_worker;
  boost::shared_ptr< ::SctApi::Trigger> worker() { return m_worker; };
 protected:
  //static std::map<boost::shared_ptr< ::SctApi::Trigger>, TriggerServer *> serverMap;

 private:
  static std::string getServerName();

  //SctApi::Trigger &worker();
};

